/* RCS: $Id$ 
 * 
 * Sample C code framework for an input helper for sudoku
 * See ATTENTION STUDENT for places that you will have to replace
 *
 * RCS: $Log$
 */
#include <stdio.h>

#define MAXSIZE 16
#define ERR_NO_MEM -1
#define LINE_MAX 80
#define KBFILE sudoku.pl

char **matrix; 

/* Allocates 2 d array for sudoku matrix
 */
int init () {
  int i; 
  matrix = (char **) malloc (sizeof(char *) * MAXSIZE); 
  if (matrix == NULL) { return ERR_NO_MEM; } 
  for (i = 0; i < MAXSIZE; i++) { 
    matrix[i] = (char *) malloc (sizeof(char) * MAXSIZE); 
    if (matrix[i] == NULL) { return ERR_NO_MEM; } 
  } 
  return 0; 
}  

/* Reads input file subject to specifications on: 
 * http://www.comp.nus.edu.sg/~kanmy/courses/3243_2006/hw-hexSudoku.html
 */
int readInput(char *filename, char *progName, int *actualDim) {
  FILE *fp;
  char c;
  int row = 0;
  int col = 0;

  fp = fopen (filename,"r");
  if (fp != NULL) {
    do {
      c = fgetc (fp);
      if (c == '\n') { *actualDim = row+1; col = 0; row++; }
      else if (c == EOF) { ; }
      else { matrix[row][col++] = c; } 
    } while (c != EOF);
    fclose (fp);
  } else { 
    fprintf (stderr, "%s fatal: can't open input file \"%s\"!\n", progName, filename);
    return 1;
  }
  return 0;
}

/* Read input the prolog output
 * Here we assume that the first letter in the variable stands for the column;
 * the second variable stands for the row
 */
int readPrologOutput (FILE *fp) {
  char buf[80];
  int bytes = 1;

  while (fgets(buf, LINE_MAX, fp) != NULL) {
    int col = buf[0]-'A';
    int row = buf[1]-'A';
    matrix[row][col] = buf[5];
  }
  fclose (fp);
  return 0;
}

/* Write the (filled-in) matrix out to standard output
 */
int writeOutput(int actualDim) {
  int i,j;

  for (i = 0; i < actualDim; i++) {
    for (j = 0; j < actualDim; j++) {
      printf ("%c", matrix[i][j]);
    }
    printf ("\n");
  }
  return 0;
}

/* Usage message, run if invoked incorrectly
 */
void usage(char *progName) {
  fprintf (stderr, "%s usage error:\n", progName);
  fprintf (stderr, "\tUse \"%s <input_file> <prolog_output>\" or \"%s <input_file> < <prolog_output>\"\n\n", 
	   progName, progName);
}

/* Main procedure :
 *  - canonicalizes input from cmd line or stdin
 *  - calls init to allocate memory
 *  - reads input
 *  - processes output 
 */
int main (int argc, char **argv) {
  FILE *fp;			/* file pointer to input file */
  int flag = 0;			/* stores status: 0 ok, !0 problem */
  int actualDim = 0;

  flag = init();		/* call init for dynamic memory allocation */

  if (argc == 3) {		/* canonicalize input */
    readInput(argv[1],argv[0], &actualDim); /* read problem input file */

    if (flag == 0) {
      fp = fopen (argv[2],"r");
      if (fp != NULL) {
      } else {
	fprintf (stderr, "%s fatal: can't open input file \"%s\"!\n", argv[0], argv[2]);
	flag = 1;
      }
    }
  } else if (argc == 2) {
    readInput(argv[1],argv[0], &actualDim); /* read problem input file */
    fp = stdin;
  } else {
    flag = 1;			/* oops */
    usage(argv[0]);
  }

  if (flag == 0) {		/* read prolog output */
    flag = readPrologOutput(fp);
  }

  if (flag == 0) {		/* write output as prolog KB */
    flag = writeOutput(actualDim);
  }
}
